#include "bnode.h"

// Use the mCount static as a global variable.  I am hiding it 
// INSIDE the CPP file for encapsulation purposes.
static int mCount = 0;

// Accessor methodes for the mCount global
void bnode::resetCompareCount(){mCount = 0;}
int  bnode::getCompareCount()  {return(mCount);}


//------------------------------------------------
//   Constructors and destructors
//------------------------------------------------

// default constructor
bnode::bnode()
{
  defaultNode();
}

bnode::bnode(int iKey)
{
  defaultNode();
  setKey(iKey);
}

bnode::bnode(bnode *iLeft, bnode *iRight, bnode *iParent, int iKey)
{
  defaultNode();

  linkLeft(iLeft);
  linkRight(iRight);
  pParent = iParent;
  setKey(iKey);
}

// destructor
bnode::~bnode()
{
  // If the parent of the node exists, link the appropriate
  // left or right link of the parent to NIL
  if (pParent)
  {      
    if (pParent->pLeft == this)
      pParent->linkLeft(NIL);
    else if (pParent->pRight == this)
      pParent->linkRight(NIL);
  }
  
  // Destroy the left and right children, if they exist
  if (pLeft)  delete pLeft;
  if (pRight) delete pRight;
}


// A helper function for defaulting all the internal members
// of the node class.
void bnode::defaultNode()
{
  pLeft   = NIL;
  pRight  = NIL;
  pParent = NIL;
  mKey    = 0;
}


//------------------------------------------------
//   Convenience methods for linking child and
// parent nodes.
//------------------------------------------------
 
//   The linkLeft and linkRight methods each
// attach the x ptr to the appropriate left
// or right link, and MAY link x's parent
// ptr back to the current node.  This upLink
// happens only if the parent exist and the
// calling routine specifies upLink = 1 (which
// is the default). 
void bnode::linkLeft(bnode *x, int upLink)
{
  pLeft = x;   
  if ( (x) && (upLink) ) x->pParent = this;
}

void bnode::linkRight(bnode *x, int upLink)
{
  pRight = x;  
  if ((x) && (upLink) )  x->pParent = this;
}


void bnode::linkParent(bnode *x)
{
  pParent = x;
}

void bnode::linkFull(bnode *left, bnode *right, bnode *parent)
{
  linkLeft(left);
  linkRight(right);
  pParent = parent;
}

//   This routine takes the current node, determines if it is the
// left or right child, then substitute's the appropriate
// parent/child links for that of the inputted y node.
void bnode::linkSubs(bnode *y)
{
  if (pParent->pLeft == this)
    pParent->linkLeft(y);
  if (pParent->pRight == this)
    pParent->linkRight(y);  
}



//------------------------------------------------
//   Methods which have to do with comparing   
// keys in the node.  Use these methods to do any
// key comparisons, and the mCount variable will
// be appropriately updated.
//
//   They each return a -1, 0, or 1:
// -1   Current node's key is LESS THAN inputted key
//  0   Current node's key is EQUAL TO inputted key
// +1   Current node's key is GREATER THAN inputted key
//------------------------------------------------

// Given a node, compare key values
int bnode::compareKeys(bnode *x)
{
  return (compareKeys(x->mKey));
}

// Given just a key, compare key values
int bnode::compareKeys(int iKey)
{
  int compVal = 0;
  
  if (mKey < iKey) compVal = -1;
  if (mKey > iKey) compVal = 1;

  mCount++;
  
  return(compVal);
}


