#include "btree.h"
#include<iostream>

using namespace std;

//------------------------------------------------
//   Constructors and destructors
//------------------------------------------------
btree::btree()
{
  pRoot      = NIL;
  mNodeCount = 0;
}

btree::~btree()
{
  if (pRoot) delete pRoot;
}



//------------------------------------------------
//------------------------------------------------
void btree::insert(int iKey)
{
  bnode *x = new bnode(iKey);
  insert(x);
}

void btree::insert(bnode *x)
{
  bnode *pLast = NIL;
  bnode *pCurr = pRoot;
  int keyFound = 0;
  int comparison;
  while (pCurr != NIL) {
    pLast = pCurr;
    comparison = x->compareKeys(pCurr);
    if (comparison > 0) pCurr = pCurr->Right();
    else                pCurr = pCurr->Left();
  }

  // rely on comparison having already been set

  mNodeCount++;

  if (pLast == NIL) pRoot = x;

  else {
    x->linkParent(pLast);
    if (comparison > 0) pLast->linkRight(x);
    else                pLast->linkLeft(x);
  }
}




//------------------------------------------------
//------------------------------------------------
bnode *btree::search(int iKey)
{
  bnode *pCurr = pRoot;
  int keyFound = 0;
  while ( (pCurr != NIL) && (!keyFound) ) {
    int comparison = pCurr->compareKeys(iKey);
    if (comparison == 0)       keyFound = 1;
    else if (comparison == -1) pCurr = pCurr->Right();
    else if (comparison == 1)  pCurr = pCurr->Left();
    else cerr << "ERROR:  compareKeys() returned something other than -1, 0 , 1" << endl;
  }

  if (!keyFound) return NIL;
  return pCurr;
}//search



//------------------------------------------------
//   Print routines for rendering the tree.  
// There is a wrapper printTree() procedure that
// relies on the recursive procedure truePrintTree()
// as well as the helper routine printNode().
//
//   The routines render the tree side-ways...so
// that right is on top and left is on bottom.
// The tree is oriented with the root on the left.
//------------------------------------------------
void btree::printTree()
{
  if (!pRoot) cout << "Tree empty!" << endl;
  else truePrintTree(pRoot,0);  
}//printTree


void btree::truePrintTree(bnode *curr, int level)
{
  if (curr != NIL)
  {     
    truePrintTree(curr->Right(),level+1);
    printNode(level,curr);  
    truePrintTree(curr->Left(),level+1);
  }//if
}//truePrintTree


void btree::printNode(int level, bnode *x)
{
  for (int i=0; i<level; i++)
    cout << "\t";
  
  cout << x->getKey() << endl;
}//printNode




//------------------------------------------------
//   A simple test function for the class.
//------------------------------------------------
void btree::test()
{
  btree bt;
 
  cout << "TESTING INSERT:" << endl;
  cout << "  Inserting:  5, 2, 6, 1, 4" << endl;
  
  bt.insert(5);
  bt.insert(2);
  bt.insert(6);
  bt.insert(1);
  bt.insert(4);
  
  cout << "  Your tree should look like this:" << endl
       << "\t6" << endl
       << "5" << endl
       << "\t\t4" << endl
       << "\t2" << endl
       << "\t\t1" << endl << endl;
  cout << "  Your tree:" << endl;
  bt.printTree();
  cout << endl;
  
  cout << "  Should have needed 6 key comparisons." << endl;  
  cout << "  Your tree needed " << bnode::getCompareCount()   
       << " comparisons." << endl;
  cout << "  You tree believes it has " << bt.getNodeCount() << " nodes in it." << endl << endl;
  // Declare some temp vars we will ndeed
  bnode *x;
  bnode *y;

  cout << "TESTING SEARCH: " << endl;  
  bnode::resetCompareCount();  
  x = bt.search(6);
  cout << "  Searing for 6: " << (x?"[found]":"[not found]") 
       <<  "  (should be found)" << endl;
  x = bt.search(8);
  cout << "  Searing for 8: " << (x?"[found]":"[not found]")
       << " (shouldn't be found)" << endl;
  
  cout << "  Should have needed 4 key comparisons." << endl;  
  cout << "  Your tree needed " << bnode::getCompareCount() 
       << " comparisons." << endl << endl;  

  bnode::resetCompareCount();  
  x = bt.search(1);
  cout << "  Searing for 1: " << (x?"[found]":"[not found]") 
       << " (should be found)" << endl;
  x = bt.search(3);
  cout << "  Searing for 3: " << (x?"[found]":"[not found]") 
       << " (shouldn't be found)" << endl;
  
  cout << "  Should have needed 6 key comparisons." << endl;  
  cout << "  Your tree needed " << bnode::getCompareCount() 
       << " comparisons." << endl << endl;  
}
