#include "bnode.h"
#include "btree.h"
#include<iostream>
#include<fstream>

using namespace std;

// ==========================================================
//  AUTHOR:  R.P. Wiegand        CREATED: 14-Mar-06
// ----------------------------------------------------------
//   This is the main program for the project.  It accepts
// three command line parameters:  
//   - file name for the insert keys
//   - file name for the search keys
//
//  The code here is simple but catches several major, obvious
// errors (no file names specified, file cannot open, etc.).
// ==========================================================


// Function prototypes for the two main operationrs:  insert and search
int insertTest(char *inFileName, btree &bt);
int searchTest(char *inFileName, btree &bt);


int main(int argc, char *argv[])
{
   // --- Command line error catching ----
  if (argc < 3)
  {
    cerr << "Incorrect usage.  You must specify two command line parameters:" << endl
         << "  - The file name for the insert data set" << endl
         << "  - The file name for the search data set" << endl 
	 <<  "  " << argv[0] << " <insertfile> <searchfile>" <<endl << endl;
    return(1);
  }
  
  cout << "Reading Inserts from " << argv[1] << endl
       << "Reading Search from " << argv[2] << endl
       << "Applying Binary Tree" << endl;
  // --- Command line error catching ----
  
 
  // Main code here:
  btree bt;

  if (!insertTest(argv[1],bt)) return(1);
  bt.printTree();  
  if (!searchTest(argv[2],bt)) return(1);
  
  return(0);  // Quit application normally
}


// This function reads the input file and inserts the
// keys into the tree.  Some unnecessary output is
// produced for debugging, etc.  A 0 or 1 is returned
// indicating whther this was successful.
int insertTest(char *inFileName, btree &bt)
{
  ifstream *infile;

  // Try to open the file for reading...
  try {infile = new ifstream(inFileName);}
  catch (...)
  {
    cerr << "ERROR:  Could not open " << inFileName << endl;    
    return(0);    
  }//catch
  if (!infile)
  {
    cerr << "ERROR:  Could not open " << inFileName << endl;    
    return(0);    
  }//catch
  
  cout << "Inserting nodes..." << endl;
  
  bnode::resetCompareCount();  
  while (!infile->eof())
  {
    // Attempt to read a key from the file...
    int x;
    try {(*infile) >> x;}
    catch (...)      
    {
      cerr << "ERROR:  Could not read integer from " << inFileName << endl;
      return(0);
    }//catch
    if (x < 0)
    {
      cerr << "ERROR:  The key value was neg, but should be pos"  << endl
           << "        Either the file has illegal values in it or" << endl
           << "        there is some problem with the file." << endl;
      return(0);	
    }
    
        
    if (infile->eof()) break;
    
    cout << "Inserting " << x << endl;    
    bt.insert(x);      
  }

  // You might choose to put this line INSIDE the while
  // loop...
  cout << "INSERT: " << bnode::getCompareCount() << " " 
       << bt.getNodeCount() << endl;
  
  infile->close();
  delete infile;  

  return(1);   //Success!
}


// This function reads the input file and searches the
// for the keys in the tree.  Some unnecessary output is
// produced for debugging, etc.  A 0 or 1 is returned
// indicating whther this was successful.
int searchTest(char *inFileName, btree &bt)
{
  ifstream *infile;

  // Try to open the file for reading...
  try {infile = new ifstream(inFileName);}
  catch (...)
  {
    cerr << "ERROR:  Could not open " << inFileName << endl;    
    return(0);    
  }//catch
  if (!infile) 
  {
    cerr << "ERROR:  Could not open " << inFileName << endl;    
    return(0);    
  }//catch

  cout << "Searching for nodes..." << endl;
  
  bnode::resetCompareCount();  
  while (!infile->eof())
  {
    // Attempt to read a key from the file...
    int x;
    try {(*infile) >> x;}
    catch (...)      
    {
      cerr << "ERROR:  Could not read integer from " << inFileName << endl;
      return(0);
    }//catc
    if (x < 0)
    {
      cerr << "ERROR:  The key value was neg, but should be pos"  << endl
           << "        Either the file has illegal values in it or" << endl
           << "        there is some problem with the file." << endl;
      return(0);	
    }

    cout << "Searching for " << x << "  ";    
    bnode *node = bt.search(x);      
    cout << (node?"[found]":"[not found]") << endl;
  }

  cout << "SEARCH: " << bnode::getCompareCount() << " " 
       << bt.getNodeCount() << endl;

  
  infile->close();
  delete infile;  

  return(1); //Success!
}
